@@ -1,8 +1,11 @@
 ;;; cplus-md.el --- C++ code editing commands for Emacs
-;;; Copyright (C) 1985, 1992, 1994, 1995 Free Software Foundation, Inc.
+
+;; Copyright (C) 1985, 1992, 1994, 1995 Free Software Foundation, Inc.
+
+;; Maintainer: Dave Detlefs <dld@cs.cmu.edu>
+;; Keywords: c
 
 ;; This file is part of GNU Emacs.
-;; Keywords: c languages oop
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -15,10 +18,9 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
-;; Maintainer: Dave Detlefs <dld@cs.cmu.edu>
-;; Keywords: c
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
 ;;; Commentary:
 
@@ -123,7 +125,19 @@
 
 (if c++-mode-syntax-table
     ()
-  (setq c++-mode-syntax-table (copy-syntax-table c-mode-syntax-table))
+  (setq c++-mode-syntax-table (make-syntax-table))
+  (modify-syntax-entry ?\\ "\\" c++-mode-syntax-table)
+  (modify-syntax-entry ?/ ". 14" c++-mode-syntax-table)
+  (modify-syntax-entry ?* ". 23" c++-mode-syntax-table)
+  (modify-syntax-entry ?+ "." c++-mode-syntax-table)
+  (modify-syntax-entry ?- "." c++-mode-syntax-table)
+  (modify-syntax-entry ?= "." c++-mode-syntax-table)
+  (modify-syntax-entry ?% "." c++-mode-syntax-table)
+  (modify-syntax-entry ?< "." c++-mode-syntax-table)
+  (modify-syntax-entry ?> "." c++-mode-syntax-table)
+  (modify-syntax-entry ?& "." c++-mode-syntax-table)
+  (modify-syntax-entry ?| "." c++-mode-syntax-table)
+  (modify-syntax-entry ?\' "\"" c++-mode-syntax-table)
   (modify-syntax-entry ?* ". 23b" c++-mode-syntax-table)
   (modify-syntax-entry ?/ ". 124" c++-mode-syntax-table)
   (modify-syntax-entry ?\n ">" c++-mode-syntax-table)
@@ -143,8 +157,102 @@ with the colon on the first line.")
   "*Indicates how far to indent an line following an empty argument
 list.  Nil indicates to just after the paren.")
 
+(defvar c++-imenu-generic-expression
+  (` 
+   ((nil
+     (, 
+      (concat
+       "^"				  ; beginning of line is required
+       "\\(template[ \t]*<[^>]+>[ \t]*\\)?" ; there may be a "template <...>"
+       "\\([a-zA-Z0-9_:]+[ \t]+\\)?"	  ; type specs; there can be no
+       "\\([a-zA-Z0-9_:]+[ \t]+\\)?"	  ; more than 3 tokens, right?
+       
+       "\\("				  ; last type spec including */&
+       "[a-zA-Z0-9_:]+"
+       "\\([ \t]*[*&]+[ \t]*\\|[ \t]+\\)"	  ; either pointer/ref sign or whitespace
+       "\\)?"				  ; if there is a last type spec
+       "\\("			      ; name; take that into the imenu entry
+       "[a-zA-Z0-9_:~]+"		      ; member function, ctor or dtor...
+					; (may not contain * because then 
+					; "a::operator char*" would become "char*"!)
+       "\\|"
+       "\\([a-zA-Z0-9_:~]*::\\)?operator"
+       "[^a-zA-Z1-9_][^(]*"	      ; ...or operator
+       " \\)"
+       "[ \t]*([^)]*)[ \t\n]*[^	      ;]" ; require something other than a ; after
+					; the (...) to avoid prototypes.  Can't
+					; catch cases with () inside the parentheses
+					; surrounding the parameters
+					; (like "int foo(int a=bar()) {...}"
+       
+       )) 6)    
+    ("Class" 
+     (, (concat 
+	 "^"				   ; beginning of line is required
+	 "\\(template[ \t]*<[^>]+>[ \t]*\\)?" ; there may be a "template <...>"
+	 "class[ \t]+"
+	 "\\([a-zA-Z0-9_]+\\)"                ; this is the string we want to get
+	 "[ \t]*[:{]"
+	 )) 2)
+;; Example of generic expression for finding prototypes, structs, unions, enums.
+;; Uncomment if you want to find these too.  It will be a bit slower gathering
+;; the indexes.
+;    ("Prototypes"
+;     (, 
+;      (concat
+;       "^"				  ; beginning of line is required
+;       "\\(template[ \t]*<[^>]+>[ \t]*\\)?" ; there may be a "template <...>"
+;       "\\([a-zA-Z0-9_:]+[ \t]+\\)?"	  ; type specs; there can be no
+;       "\\([a-zA-Z0-9_:]+[ \t]+\\)?"	  ; more than 3 tokens, right?
+       
+;       "\\("				  ; last type spec including */&
+;       "[a-zA-Z0-9_:]+"
+;       "\\([ \t]*[*&]+[ \t]*\\|[ \t]+\\)"	  ; either pointer/ref sign or whitespace
+;       "\\)?"				  ; if there is a last type spec
+;       "\\("			      ; name; take that into the imenu entry
+;       "[a-zA-Z0-9_:~]+"		      ; member function, ctor or dtor...
+;					; (may not contain * because then 
+;					; "a::operator char*" would become "char*"!)
+;       "\\|"
+;       "\\([a-zA-Z0-9_:~]*::\\)?operator"
+;       "[^a-zA-Z1-9_][^(]*"	      ; ...or operator
+;       " \\)"
+;       "[ \t]*([^)]*)[ \t\n]*;" 	; require ';' after
+;					; the (...) Can't
+;					; catch cases with () inside the parentheses
+;					; surrounding the parameters
+;					; (like "int foo(int a=bar());"       
+;       )) 6)    
+;    ("Struct"
+;     (, (concat
+;	 "^"				; beginning of line is required
+;	 "\\(static[ \t]+\\)?"		; there may be static or const.
+;	 "\\(const[ \t]+\\)?"
+;	 "struct[ \t]+"
+;	 "\\([a-zA-Z0-9_]+\\)"		; this is the string we want to get
+;	 "[ \t]*[{]"
+;	 )) 3)
+;    ("Enum"
+;     (, (concat
+;	 "^"				; beginning of line is required
+;	 "\\(static[ \t]+\\)?"		; there may be static or const.
+;	 "\\(const[ \t]+\\)?"
+;	 "enum[ \t]+"
+;	 "\\([a-zA-Z0-9_]+\\)"		; this is the string we want to get
+;	 "[ \t]*[{]"
+;	 )) 3)
+;    ("Union"
+;     (, (concat
+;	 "^"				; beginning of line is required
+;	 "\\(static[ \t]+\\)?"		; there may be static or const.
+;	 "\\(const[ \t]+\\)?"
+;	 "union[ \t]+"
+;	 "\\([a-zA-Z0-9_]+\\)"		; this is the string we want to get
+;	 "[ \t]*[{]"
+;	 )) 3)
+    ))
+  "Imenu generic expression for C++ mode.  See `imenu-generic-expression'.")
 
-;;;###autoload
 (defun c++-mode ()
   "Major mode for editing C++ code.  Very much like editing C code.
 Expression and list commands understand all C++ brackets.
@@ -182,7 +290,7 @@ Variables controlling indentation style:
     Extra indentation for line that is a label, or case or ``default:'', or
     ``public:'' or ``private:'', or ``protected:''.
  c++-electric-colon
-    If non-nil at invocation of c++-mode (t is the default) colon electricly
+    If non-nil at invocation of c++-mode (t is the default) colon electrically
     indents.
  c++-empty-arglist-indent
     If non-nil, a function declaration or invocation which ends a line with a
@@ -212,6 +320,8 @@ Turning on C++ mode calls the value of the variable `c++-mode-hook' with
 no args if that value is non-nil."
   (interactive)
   (kill-all-local-variables)
+  ;; This code depends on the old C mode.
+  (require 'c-mode)
   (use-local-map c++-mode-map)
   (set-syntax-table c++-mode-syntax-table)
   (setq major-mode 'c++-mode
@@ -228,6 +338,8 @@ no args if that value is non-nil."
   (set (make-local-variable 'paragraph-ignore-fill-prefix) t)
   (set (make-local-variable 'require-final-newline) t)
   (set (make-local-variable 'parse-sexp-ignore-comments) t)
+  (make-local-variable 'imenu-generic-expression)
+  (setq imenu-generic-expression c++-imenu-generic-expression)
   (run-hooks 'c++-mode-hook)
   (if c++-electric-colon
       (define-key c++-mode-map ":" 'electric-c++-terminator)))
@@ -301,15 +413,15 @@ no args if that value is non-nil."
 			;; So quickly rule out most other uses of colon
 			;; and do no indentation for them.
 			(and (eq last-command-char ?:)
-			     (not (looking-at "case[ \t]"))
-			     (save-excursion
-			       (forward-word 1)
-			       (skip-chars-forward " \t")
-			       (< (point) end))
-			     ;; Do re-indent double colons
-			     (save-excursion
-			       (end-of-line 1)
-			       (looking-at ":")))
+			     (or (not (or (looking-at "case[ \t]")
+					  (save-excursion
+					    (forward-word 1)
+					    (skip-chars-forward " \t")
+					    (>= (point) end))))
+				 ;; Do re-indent double colons
+				 (save-excursion
+				   (end-of-line 1)
+				   (looking-at ":"))))
 			(progn
 			  (beginning-of-defun)
 			  (let ((pps (parse-partial-sexp (point) end)))
@@ -466,6 +578,8 @@ Returns nil if line starts inside a string, t if in a comment."
 		     (backward-char 1))
 		 (if (= (preceding-char) ?})
 		     0
+		   (if (= (preceding-char) ?\))
+		       (forward-list -1))
 		   (beginning-of-line)	; continued arg decls or member inits
 		   (skip-chars-forward " \t")
 		   (if (= (following-char) ?:)
@@ -511,7 +625,14 @@ Returns nil if line starts inside a string, t if in a comment."
 	     ;; Statement.  Find previous non-comment character.
 	     (goto-char indent-point)
 	     (c++-backward-to-noncomment containing-sexp)
-	     (if (not (memq (preceding-char) '(nil ?\, ?\; ?} ?: ?\{)))
+	     (if (and (not (memq (preceding-char) '(0 ?\, ?\; ?\} ?\{)))
+		      ;; But don't treat a line with a close-brace
+		      ;; as a continuation.  It is probably the
+		      ;; end of an enum type declaration.
+		      (save-excursion
+			(goto-char indent-point)
+			(skip-chars-forward " \t")
+			(not (= (following-char) ?}))))
 		 ;; This line is continuation of preceding line's statement;
 		 ;; indent  c-continued-statement-offset  more than the
 		 ;; previous line of the statement.
@@ -682,7 +803,7 @@ Returns nil if line starts inside a string, t if in a comment."
 		(if (= (char-after (car contain-stack)) ?{)
 		    (save-excursion
 		      (goto-char (car contain-stack))
-		      (setq val (+ c-indent-level (current-column))))
+		      (setq val (calculate-c-indent-after-brace)))
 		  (setq val (calculate-c++-indent
 			     (if (car indent-stack)
 				 (- (car indent-stack))))))
