@@ -49,7 +49,10 @@ may contain even `F', `b', `i' and `s'.  See also the variable
 ;;;###autoload
 (defvar dired-chown-program
   (if (memq system-type '(hpux dgux usg-unix-v irix linux lignux))
-      "chown" "/etc/chown")
+      "chown"
+    (if (file-exists-p "/usr/sbin/chown")
+	"/usr/sbin/chown"
+      "/etc/chown"))
   "Name of chown command (usually `chown' or `/etc/chown').")
 
 (defvar dired-chmod-program
