@@ -1,6 +1,7 @@
 ;;; doctor.el --- psychological help for frustrated users.
+;;; (censored version--see below)
 
-;; Copyright (C) 1985, 1987, 1994 Free Software Foundation, Inc.
+;; Copyright (C) 1985, 1987, 1994, 1996 Free Software Foundation, Inc.
 
 ;; Maintainer: FSF
 ;; Keywords: games
@@ -18,8 +19,9 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
 ;;; Commentary:
 
@@ -27,6 +29,16 @@
 ;; phrase-production techniques similar to the classic ELIZA demonstration
 ;; of pseudo-AI.
 
+;; This file has been censored by the Communications Decency Act.
+;; Some of its features were removed.  The law was promoted as a ban
+;; on pornography, but it bans far more than that.  The doctor program
+;; did not contain pornography, but part of it was prohibited
+;; nonetheless.
+
+;; For information on US government censorship of the Internet, and
+;; what you can do to bring back freedom of the press, see the web
+;; site http://www.vtw.org/
+
 ;;; Code:
 
 (defun doctor-cadr (x) (car (cdr x)))
@@ -67,6 +79,8 @@ reads the sentence before point, and prints the Doctor's answer."
   (setq mode-name "Doctor")
   (turn-on-auto-fill)
   (doctor-type '(i am the psychotherapist \.
+		 for your protection, i have been censored according to
+		 the Communications Decency Act \.
 		 ($ please) ($ describe) your ($ problems) \.
 		 each time you are finished talking, type \R\E\T twice \.))
   (insert "\n"))
@@ -223,7 +237,7 @@ reads the sentence before point, and prints the Doctor's answer."
   (make-local-variable 'feelings-about)
   (setq feelings-about
 	'((feelings about)
-	  (aprehensions toward)
+	  (apprehensions toward)
 	  (thoughts on)
 	  (emotions toward)))
   (make-local-variable 'random-adjective)
@@ -602,7 +616,6 @@ reads the sentence before point, and prints the Doctor's answer."
 (doctor-put-meaning ibm 'mach)
 (doctor-put-meaning pc 'mach)
 (doctor-put-meaning bitching 'foul)
-(doctor-put-meaning shit 'foul)
 (doctor-put-meaning bastard 'foul)
 (doctor-put-meaning damn 'foul)
 (doctor-put-meaning damned 'foul)
@@ -677,7 +690,6 @@ reads the sentence before point, and prints the Doctor's answer."
 (doctor-put-meaning lonely 'mood)
 (doctor-put-meaning angry 'mood)
 (doctor-put-meaning mad 'mood)
-(doctor-put-meaning pissed 'mood)
 (doctor-put-meaning jealous 'mood)
 (doctor-put-meaning afraid 'fear)
 (doctor-put-meaning terrified 'fear)
@@ -691,8 +703,6 @@ reads the sentence before point, and prints the Doctor's answer."
 (doctor-put-meaning cocks 'sexnoun)
 (doctor-put-meaning dick 'sexnoun)
 (doctor-put-meaning dicks 'sexnoun)
-(doctor-put-meaning cunt 'sexnoun)
-(doctor-put-meaning cunts 'sexnoun)
 (doctor-put-meaning prostitute 'sexnoun)
 (doctor-put-meaning condom 'sexnoun)
 (doctor-put-meaning sex 'sexnoun)
@@ -750,18 +760,14 @@ reads the sentence before point, and prints the Doctor's answer."
 (doctor-put-meaning wine 'alcohol)
 (doctor-put-meaning whiskey 'alcohol)
 (doctor-put-meaning scotch 'alcohol)
-(doctor-put-meaning fuck 'sexverb)
-(doctor-put-meaning fucked 'sexverb)
 (doctor-put-meaning screw 'sexverb)
 (doctor-put-meaning screwing 'sexverb)
-(doctor-put-meaning fucking 'sexverb)
 (doctor-put-meaning rape 'sexverb)
 (doctor-put-meaning raped 'sexverb)
 (doctor-put-meaning kiss 'sexverb)
 (doctor-put-meaning kissing 'sexverb)
 (doctor-put-meaning kisses 'sexverb)
 (doctor-put-meaning screws 'sexverb)
-(doctor-put-meaning fucks 'sexverb)
 (doctor-put-meaning because 'conj)
 (doctor-put-meaning but 'conj)
 (doctor-put-meaning however 'conj)
@@ -879,7 +885,7 @@ Otherwise call the Doctor to parse preceding sentence."
 (defun doctor-doc (sent)
   (cond
    ((equal sent '(foo))
-    (doctor-type '(bar! ($ please)($ continue))))
+    (doctor-type '(bar! ($ please)($ continue) \.)))
    ((member sent howareyoulst)
     (doctor-type '(i\'m ok \.  ($ describe) yourself \.)))
    ((or (member sent '((good bye) (see you later) (i quit) (so long)
@@ -1075,8 +1081,7 @@ the subject noun, and return the portion of the sentence following it."
 		expect expected expects expel expels expelled
 		explain explained explains
 		fart farts feel feels felt fight fights find finds finding
-		forget forgets forgot fought found fuck fucked
-		fucking fucks
+		forget forgets forgot fought found
 		gave get gets getting give gives go goes going gone got gotten
 		had harm harms has hate hated hates have having
 		hear heard hears hearing help helped helping helps
@@ -1217,7 +1222,7 @@ the subject noun, and return the portion of the sentence following it."
   (memq x '(all also always amusing any anyway associated awesome
 		bad beautiful best better but certain clear
 		ever every fantastic fun funny
-		good great gross growdy however if ignorant
+		good great grody gross however if ignorant
 		less linked losing lusing many more much
 		never nice obnoxious often poor pretty real related rich
 		similar some stupid super superb
@@ -1620,7 +1625,7 @@ Hack on previous word, setting global variable OWNER to correct result."
 
 (defun doctor-strangelove ()
   (interactive)
-  (insert "Mein fuhrer!!\n")
+  (insert "Mein fuehrer!!\n")
   (doctor-read-print))
 
 ;;; doctor.el ends here
