@@ -18,12 +18,13 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
 ;;; Commentary:
 
-;; Utility functions for maol and netnews handling.  These handle fine
+;; Utility functions for mail and netnews handling.  These handle fine
 ;; points of header parsing.
 
 ;;; Code:
@@ -38,6 +39,17 @@
 Otherwise, (the default) use a smaller, somewhat faster, and
 often correct parser.")
 
+;; Returns t if file FILE is an Rmail file.
+;;;###autoload
+(defun mail-file-babyl-p (file)
+  (let ((buf (generate-new-buffer " *rmail-file-p*")))
+    (unwind-protect
+	(save-excursion
+	  (set-buffer buf)
+	  (insert-file-contents file nil 0 100)
+	  (looking-at "BABYL OPTIONS:"))
+      (kill-buffer buf))))
+
 (defun mail-string-delete (string start end)
   "Returns a string containing all of STRING except the part
 from START (inclusive) to END (exclusive)."
@@ -63,7 +75,7 @@ Return a modified address list."
 					      (match-end 0))))
 
        ;; Detect nested comments.
-       (if (string-match "[ \t]*(\\([^)\"\\]\\|\\\\.\\|\\\\\n\\)*(" address)
+       (if (string-match "[ \t]*(\\([^)\\]\\|\\\\.\\|\\\\\n\\)*(" address)
 	   ;; Strip nested comments.
 	   (save-excursion
 	     (set-buffer (get-buffer-create " *temp*"))
@@ -86,7 +98,7 @@ Return a modified address list."
 	 (while (setq pos (string-match 
 			    ;; This doesn't hack rfc822 nested comments
 			    ;;  `(xyzzy (foo) whinge)' properly.  Big deal.
-			    "[ \t]*(\\([^)\"\\]\\|\\\\.\\|\\\\\n\\)*)"
+			    "[ \t]*(\\([^)\\]\\|\\\\.\\|\\\\\n\\)*)"
 			    address))
 	   (setq address
 		 (mail-string-delete address
@@ -105,7 +117,7 @@ Return a modified address list."
 		 (mail-string-delete address
 				     pos (match-end 0)))))
        ;; Retain only part of address in <> delims, if there is such a thing.
-       (while (setq pos (string-match "\\(,\\|\\`\\)[^,]*<\\([^>,]*>\\)"
+       (while (setq pos (string-match "\\(,\\s-*\\|\\`\\)[^,]*<\\([^>,]*>\\)"
 				      address))
 	 (let ((junk-beg (match-end 1))
 	       (junk-end (match-beginning 2))
@@ -128,7 +140,7 @@ Usenet paths ending in an element that matches are removed also."
 	    (concat (if rmail-default-dont-reply-to-names
 			(concat rmail-default-dont-reply-to-names "\\|")
 		        "")
-		    (concat (regexp-quote (user-original-login-name))
+		    (concat (regexp-quote (user-login-name))
 			    "\\>"))))
   (let ((match (concat "\\(^\\|,\\)[ \t\n]*\\([^,\n]*!\\|\\)\\("
 		       rmail-dont-reply-to-names
@@ -136,9 +148,10 @@ Usenet paths ending in an element that matches are removed also."
 	(case-fold-search t)
 	pos epos)
     (while (setq pos (string-match match userids))
-      (if (> pos 0) (setq pos (1+ pos)))
+      (if (> pos 0) (setq pos (match-beginning 2)))
       (setq epos
-	    (if (string-match "[ \t\n,]+" userids (match-end 0))
+	    ;; Delete thru the next comma, plus whitespace after.
+	    (if (string-match ",[ \t\n]+" userids (match-end 0))
 		(match-end 0)
 	      (length userids)))
       (setq userids
@@ -152,6 +165,7 @@ Usenet paths ending in an element that matches are removed also."
 	(substring userids (match-end 0))
       userids)))
 
+;;;###autoload
 (defun mail-fetch-field (field-name &optional last all)
   "Return the value of the header field FIELD-NAME.
 The buffer is expected to be narrowed to just the headers of the message.
@@ -161,16 +175,19 @@ If third arg ALL is non-nil, concatenate all such fields with commas between."
     (goto-char (point-min))
     (let ((case-fold-search t)
 	  (name (concat "^" (regexp-quote field-name) "[ \t]*:[ \t]*")))
-      (goto-char (point-min))
       (if all
 	  (let ((value ""))
 	    (while (re-search-forward name nil t)
 	      (let ((opoint (point)))
 		(while (progn (forward-line 1)
 			      (looking-at "[ \t]")))
+		;; Back up over newline, then trailing spaces or tabs
+		(forward-char -1)
+		(skip-chars-backward " \t" opoint)
 		(setq value (concat value
 				    (if (string= value "") "" ", ")
-				    (buffer-substring opoint (1- (point)))))))
+				    (buffer-substring-no-properties
+				     opoint (point))))))
 	    (and (not (string= value "")) value))
 	(if (re-search-forward name nil t)
 	    (progn
@@ -178,7 +195,10 @@ If third arg ALL is non-nil, concatenate all such fields with commas between."
 	      (let ((opoint (point)))
 		(while (progn (forward-line 1)
 			      (looking-at "[ \t]")))
-		(buffer-substring opoint (1- (point))))))))))
+		;; Back up over newline, then trailing spaces or tabs
+		(forward-char -1)
+		(skip-chars-backward " \t" opoint)
+		(buffer-substring-no-properties opoint (point)))))))))
 
 ;; Parse a list of tokens separated by commas.
 ;; It runs from point to the end of the visible part of the buffer.
