@@ -163,8 +163,7 @@ This is normally copied from `default-directory' when Emacs starts.")
   '(("-bw" 1 x-handle-numeric-switch border-width)
     ("-d" 1 x-handle-display)
     ("-display" 1 x-handle-display)
-    ("-name" 1 x-handle-name-rn-switch)
-    ("-rn" 1 x-handle-name-rn-switch)
+    ("-name" 1 x-handle-name-switch)
     ("-title" 1 x-handle-switch title)
     ("-T" 1 x-handle-switch title)
     ("-r" 0 x-handle-switch reverse t)
@@ -191,7 +190,7 @@ This is normally copied from `default-directory' when Emacs starts.")
     ("-bd" 1 x-handle-switch)
     ("--border-width" 1 x-handle-numeric-switch border-width)
     ("--display" 1 x-handle-display)
-    ("--name" 1 x-handle-name-rn-switch)
+    ("--name" 1 x-handle-name-switch)
     ("--title" 1 x-handle-switch title)
     ("--reverse-video" 0 x-handle-switch reverse t)
     ("--font" 1 x-handle-switch font)
@@ -305,7 +304,7 @@ from being initialized.")
 
 ;; This function is called from the subdirs.el file.
 (defun normal-top-level-add-to-load-path (dirs)
-  (let ((tail (member default-directory load-path)))
+  (let ((tail (member (directory-file-name default-directory) load-path)))
     (setcdr tail (append (mapcar 'expand-file-name dirs) (cdr tail)))))
 
 (defun normal-top-level ()
