@@ -1,26 +1,26 @@
 ;; sun.el --- keybinding for standard default sunterm keys
 
+;; Copyright (C) 1987 Free Software Foundation, Inc.
+
 ;; Author: Jeff Peck <peck@sun.com>
 ;; Keywords: terminals
 
-;; Copyright (C) 1987 Free Software Foundation, Inc.
-
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY.  No author or distributor
-;; accepts responsibility to anyone for the consequences of using it
-;; or for whether it serves any particular purpose or works at all,
-;; unless he says so in writing.  Refer to the GNU Emacs General Public
-;; License for full details.
+;; GNU Emacs is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation; either version 2, or (at your option)
+;; any later version.
 
-;; Everyone is granted permission to copy, modify and redistribute
-;; GNU Emacs, but only under the conditions described in the
-;; GNU Emacs General Public License.   A copy of this license is
-;; supposed to have been given to you along with GNU Emacs so you
-;; can know your rights and responsibilities.  It should be in a
-;; file named COPYING.  Among other things, the copyright notice
-;; and this notice must be preserved on all copies.
+;; GNU Emacs is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+
+;; You should have received a copy of the GNU General Public License
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
 ;;; Commentary:
 
@@ -99,8 +99,8 @@
 (defvar sun-esc-bracket nil
   "*If non-nil, rebind ESC [ as prefix for Sun function keys.")
 
-(define-prefix-command 'sun-raw-prefix)
-(define-key function-key-map "\e[" 'sun-raw-prefix)
+(defvar sun-raw-prefix (make-sparse-keymap))
+(define-key function-key-map "\e[" sun-raw-prefix)
 
 (define-key sun-raw-prefix "210z" [r3])
 (define-key sun-raw-prefix "213z" [r6])
