@@ -1,11 +1,12 @@
-;;;; texinfo.el--major mode for editing Texinfo files.
-;; Copyright (C) 1985, '88, '89, 
-;;                '90, '91, '92, '93 Free Software Foundation, Inc.
+;;; texinfo.el --- major mode for editing Texinfo files
 
-;;; Author: Robert J. Chassell          
-;;; Maintainer: FSF
+;; Copyright (C) 1985, 1988, 1989, 1990, 1991, 1992, 1993 Free Software
+;; Foundation, Inc.
 
-;;; This file is part of GNU Emacs.
+;; Author: Robert J. Chassell          
+;; Maintainer: FSF
+
+;; This file is part of GNU Emacs.
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -18,8 +19,9 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
 
 ;;; Autoloads:
@@ -91,7 +93,7 @@ menus in the buffer (incorporating descriptions from pre-existing
 menus) before it constructs the master menu.
 
 The function removes the detailed part of an already existing master
-menu.  This action depends on the pre-exisitng master menu using the
+menu.  This action depends on the pre-existing master menu using the
 standard `texinfo-master-menu-header'.
 
 The master menu has the following format, which is adapted from the
@@ -196,6 +198,65 @@ chapter."
   (modify-syntax-entry ?} "){" texinfo-mode-syntax-table)
   (modify-syntax-entry ?\' "w" texinfo-mode-syntax-table))
 
+;; Written by Wolfgang Bangerth <zcg51122@rpool1.rus.uni-stuttgart.de>
+;; To override this example, set either `imenu-generic-expression'
+;; or `imenu-create-index-function'.
+(defvar texinfo-imenu-generic-expression
+  '((nil "^@node[ \t]+\\([^,\n]*\\)" 1)
+    ("Chapters" "^@chapter[ \t]+\\(.*\\)$" 1))
+
+  "Imenu generic expression for TexInfo mode.  See `imenu-generic-expression'.")
+
+(defvar texinfo-font-lock-keywords
+  '(;; All but the first 2 had an OVERRIDE of t.
+    ;; It didn't seem to be any better, and it's slower--simon.
+    ("^\\(@c\\|@comment\\)\\>.*" . font-lock-comment-face) 	;comments
+    ;; Robert J. Chassell <bob@gnu.ai.mit.edu> says remove this line.
+    ;("\\$\\([^$]*\\)\\$" 1 font-lock-string-face t)
+    ("@\\([a-zA-Z]+\\|[^ \t\n]\\)" 1 font-lock-keyword-face)	;commands
+    ("^\\*\\(.*\\)[\t ]*$" 1 font-lock-function-name-face t) 	;menu items
+    ("@\\(emph\\|strong\\|b\\|i\\){\\([^}]+\\)" 2 font-lock-comment-face)
+    ("@\\(file\\|kbd\\|key\\){\\([^}]+\\)" 2 font-lock-string-face)
+    ("@\\(samp\\|code\\|var\\|math\\){\\([^}]+\\)"
+     2 font-lock-variable-name-face)
+    ("@\\(cite\\|xref\\|pxref\\){\\([^}]+\\)" 2 font-lock-reference-face)
+    ("@\\(end\\|itemx?\\) +\\(.+\\)" 2 font-lock-function-name-face keep)
+    )
+  "Additional expressions to highlight in TeXinfo mode.")
+
+(defvar texinfo-section-list
+  '(("top" 1)
+    ("majorheading" 1)
+    ("chapter" 2)
+    ("unnumbered" 2)
+    ("appendix" 2)
+    ("chapheading" 2)
+    ("section" 3)
+    ("unnumberedsec" 3)
+    ("appendixsec" 3)
+    ("heading" 3)
+    ("subsection" 4)
+    ("unnumberedsubsec" 4)
+    ("appendixsubsec" 4)
+    ("subheading" 4)
+    ("subsubsection" 5)
+    ("unnumberedsubsubsec" 5)
+    ("appendixsubsubsec" 5)
+    ("subsubheading" 5))
+  "Alist of sectioning commands and their relative level.")
+
+(defun texinfo-outline-level ()
+  ;; Calculate level of current texinfo outline heading.
+  (save-excursion
+    (if (bobp)
+	0
+      (forward-char 1)
+      (let* ((word (buffer-substring-no-properties 
+		    (point) (progn (forward-word 1) (point))))
+	     (entry (assoc word texinfo-section-list)))
+	(if entry
+	    (nth 1 entry)
+	  5)))))
 
 ;;; Keybindings
 (defvar texinfo-mode-map nil)
@@ -248,7 +309,6 @@ chapter."
   (define-key texinfo-mode-map "\C-c\C-s"     'texinfo-show-structure)
 
   (define-key texinfo-mode-map "\C-c}"    	'up-list)
-  (define-key texinfo-mode-map "\C-c]"    	'up-list)
   (define-key texinfo-mode-map "\C-c{"    	'texinfo-insert-braces)
 
   ;; bindings for inserting strings
@@ -360,9 +420,9 @@ value of texinfo-mode-hook."
   (make-local-variable 'indent-tabs-mode)
   (setq indent-tabs-mode nil)
   (make-local-variable 'paragraph-separate)
-  (setq paragraph-separate (concat "^\b\\|^@[a-zA-Z]*[ \n]\\|" paragraph-separate))
+  (setq paragraph-separate (concat "\b\\|@[a-zA-Z]*[ \n]\\|" paragraph-separate))
   (make-local-variable 'paragraph-start)
-  (setq paragraph-start (concat "^\b\\|^@[a-zA-Z]*[ \n]\\|" paragraph-start))
+  (setq paragraph-start (concat "\b\\|@[a-zA-Z]*[ \n]\\|" paragraph-start))
   (make-local-variable 'fill-column)
   (setq fill-column 72)
   (make-local-variable 'comment-start)
@@ -371,6 +431,17 @@ value of texinfo-mode-hook."
   (setq comment-start-skip "@c +")
   (make-local-variable 'words-include-escapes)
   (setq words-include-escapes t)
+  (make-local-variable 'imenu-generic-expression)
+  (setq imenu-generic-expression texinfo-imenu-generic-expression)
+  (make-local-variable 'font-lock-defaults)
+  (setq font-lock-defaults '(texinfo-font-lock-keywords t))
+  (make-local-variable 'outline-regexp)
+  (setq outline-regexp 
+	(concat "@\\("
+		(mapconcat 'car texinfo-section-list "\\>\\|")
+		"\\>\\)"))
+  (make-local-variable 'outline-level)
+  (setq outline-level 'texinfo-outline-level)
   (make-local-variable 'tex-start-of-header)
   (setq tex-start-of-header "%**start")
   (make-local-variable 'tex-end-of-header)
@@ -381,17 +452,14 @@ value of texinfo-mode-hook."
 ;;; Insert string commands
 
 (defconst texinfo-environment-regexp
-  "^@\\(f?table\\|enumerate\\|itemize\\|ifinfo\\|iftex\\|ifset\\|ifclear\
+  "^[ \t]*@\\(f?table\\|enumerate\\|itemize\
+\\|ifhtml\\|ifinfo\\|iftex\\|ifset\\|ifclear\
 \\|example\\|quotation\\|lisp\\|smallexample\\|smalllisp\\|display\\|format\
-\\|flushleft\\|flushright\\|ignore\\|group\\|tex\\|cartouche\\|end\
-\\|def[a-z]*\\)"
+\\|flushleft\\|flushright\\|ignore\\|group\\|tex\\|html\\|cartouche\\|menu\
+\\|titlepage\\|end\\|def[a-z]*[a-wyz]\\>\\)"
   "Regexp for environment-like Texinfo list commands.
 Subexpression 1 is what goes into the corresponding `@end' statement.")
 
-;; The following texinfo-insert-@end command not only inserts a SPC
-;; after the @end, but tries to find out what belongs there.  It is
-;; not very smart: it does not understand nested lists.
-
 (defun texinfo-insert-@end ()
   "Insert the matching `@end' for the last Texinfo command that needs one."
   (interactive)
@@ -403,9 +471,10 @@ Subexpression 1 is what goes into the corresponding `@end' statement.")
 	    (setq depth (1+ depth))
 	  (setq depth (1- depth)))))
       (looking-at texinfo-environment-regexp)
-      (setq string
-	    (buffer-substring (match-beginning 1)
-			      (match-end 1))))
+      (if (zerop depth)
+	  (setq string
+		(buffer-substring (match-beginning 1)
+				  (match-end 1)))))
     (insert "@end ")
     (if string (insert string "\n"))))
 
@@ -516,6 +585,12 @@ The default is not to surround any existing words with the braces."
 
 ;;; Texinfo file structure
 
+;; These are defined in tenfo-upd.el.
+(defvar texinfo-section-types-regexp)
+(defvar texinfo-section-level-regexp)
+(defvar texinfo-subsection-level-regexp)
+(defvar texinfo-subsubsection-level-regexp)
+
 (defun texinfo-show-structure (&optional nodes-too) 
   "Show the structure of a Texinfo file.
 List the lines in the file that begin with the @-sign commands for
