@@ -15,7 +15,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.
 
 hacked on by jwz@lucid.com 17-jun-91
   o  added a compile-time switch to turn on simple sanity checking;
@@ -251,8 +252,9 @@ Lisp_Object Qbytecode;
 
 DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
   "Function used internally in byte-compiled code.\n\
-The first argument is a string of byte code; the second, a vector of constants;\n\
-the third, the maximum stack depth used in this function.\n\
+The first argument, BYTESTR, is a string of byte code;\n\
+the second, VECTOR, a vector of constants;\n\
+the third, MAXDEPTH, the maximum stack depth used in this function.\n\
 If the third argument is incorrect, Emacs may crash.")
   (bytestr, vector, maxdepth)
      Lisp_Object bytestr, vector, maxdepth;
@@ -473,14 +475,14 @@ If the third argument is incorrect, Emacs may crash.")
 
 	case BRgoto:
 	  QUIT;
-	  pc += *pc - 127;
+	  pc += (int) *pc - 127;
 	  break;
 
 	case BRgotoifnil:
 	  if (NILP (POP))
 	    {
 	      QUIT;
-	      pc += *pc - 128;
+	      pc += (int) *pc - 128;
 	    }
 	  pc++;
 	  break;
@@ -489,7 +491,7 @@ If the third argument is incorrect, Emacs may crash.")
 	  if (!NILP (POP))
 	    {
 	      QUIT;
-	      pc += *pc - 128;
+	      pc += (int) *pc - 128;
 	    }
 	  pc++;
 	  break;
@@ -758,7 +760,18 @@ If the third argument is incorrect, Emacs may crash.")
 	  v2 = POP; v1 = TOP;
 	  CHECK_NUMBER_OR_FLOAT_COERCE_MARKER (v1, 0);
 	  CHECK_NUMBER_OR_FLOAT_COERCE_MARKER (v2, 0);
-	  TOP = (XFLOATINT (v1) == XFLOATINT (v2)) ? Qt : Qnil;
+#ifdef LISP_FLOAT_TYPE
+	  if (FLOATP (v1) || FLOATP (v2))
+	    {
+	      double f1, f2;
+
+	      f1 = (FLOATP (v1) ? XFLOAT (v1)->data : XINT (v1));
+	      f2 = (FLOATP (v2) ? XFLOAT (v2)->data : XINT (v2));
+	      TOP = (f1 == f2 ? Qt : Qnil);
+	    }
+	  else
+#endif
+	    TOP = (XINT (v1) == XINT (v2) ? Qt : Qnil);
 	  break;
 
 	case Bgtr:
