@@ -15,12 +15,14 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.  */
 
 /* Contributed by Ian Lance Taylor, ian@cygnus.com.  */
 
 /* SCO is sort of like SVR3.  */
 #include "usg5-3.h"
+#define SCO_R4
 
 #if 0 /* Turned off rather than make the Lisp code check for this. -- rms.
 	 I am assuming that (at least most of) the tests for usg-unix-v
@@ -44,9 +46,6 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 /* SCO has termios.  */
 #define HAVE_TERMIOS
 
-/* SCO has timeval.  */
-#define HAVE_TIMEVAL
-
 /* SCO has ptys with unusual names.  */
 #define HAVE_PTYS
 
@@ -64,6 +63,11 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #define HAVE_SOCKETS
 #endif
 
+/* Must use 'cc' to link when build with motif toolkit. */
+#ifndef __GNUC__
+#define LINKER cc
+#endif
+
 /* This is safe since we already assumed HAVE_SOCKET
    if using X windows.  */
 #undef LIBX11_SYSTEM
@@ -129,3 +133,9 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* wjs@wang.com (William Smith) says this is needed on 3.2.4.2.  */
 #define POSIX_SIGNALS
+
+/* wjs@wiis.wang.com says SCO 3.2 v4.2 "has sockets",
+   but only for network connections.
+   It doesn't have the kind of sockets that emacsclient.c
+   and emacsserver.c would use.  */
+#define NO_SOCKETS_IN_FILE_SYSTEM
