@@ -28,6 +28,7 @@
 #define UNEXEC	unexsunos4.o
 #define RUN_TIME_REMAP
 #define ORDINARY_LINK
+#define SUNOS4_SHARED_LIBRARIES
 
 #undef LD_SWITCH_SYSTEM
 
@@ -43,3 +44,15 @@
 #define memmove(to, from, size) bcopy (from, to, size)
 
 #undef USE_DL_STUBS
+
+#ifndef HAVE_X11R6
+/* With X11R5 it was reported that linking -lXmu dynamically
+   did not work.  With X11R6, it does work; and since normally
+   only the dynamic libraries are available, we should use them.  */
+#ifdef __GNUC__
+#define LIBXMU -Xlinker -Bstatic -lXmu -Xlinker -Bdynamic
+#else
+#define LIBXMU -Bstatic -lXmu -Bdynamic
+#endif
+
+#endif  /* not HAVE_X11R6 */
