@@ -15,10 +15,21 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.  */
 
 /* Emacs 19 face widget ported by Fred Pierresteguy */
 
+/* This file has been censored by the Communications Decency Act.
+   That law was passed under the guise of a ban on pornography, but
+   it bans far more than that.  This file did not contain pornography,
+   but it was censored nonetheless.
+
+   For information on US government censorship of the Internet, and
+   what you can do to bring back freedom of the press, see the web
+   site http://www.vtw.org/
+   */
+
 #include <config.h>
 #include <stdio.h>
 #include "lisp.h"
@@ -169,8 +180,8 @@ get_default_char_pixel_size (ew, pixel_width, pixel_height)
      int* pixel_height;
 {
   struct frame* f = ew->emacs_frame.frame;
-  *pixel_width = FONT_WIDTH (f->display.x->font);
-  *pixel_height = f->display.x->line_height;
+  *pixel_width = FONT_WIDTH (f->output_data.x->font);
+  *pixel_height = f->output_data.x->line_height;
 }
 
 static void
@@ -182,8 +193,8 @@ pixel_to_char_size (ew, pixel_width, pixel_height, char_width, char_height)
      int* char_height;
 {
   struct frame* f = ew->emacs_frame.frame;
-  *char_width = PIXEL_TO_CHAR_WIDTH (f, pixel_width);
-  *char_height = PIXEL_TO_CHAR_HEIGHT (f, pixel_height);
+  *char_width = PIXEL_TO_CHAR_WIDTH (f, (int) pixel_width);
+  *char_height = PIXEL_TO_CHAR_HEIGHT (f, (int) pixel_height);
 }
 
 static void
@@ -311,7 +322,7 @@ set_frame_size (ew)
   
   if (! XtIsSubclass (wmshell, shellWidgetClass)) abort ();
 
-  /* We don't need this for the momment. The geometry is computed in 
+  /* We don't need this for the moment. The geometry is computed in 
      xfns.c.  */
 #if 0
   /* If the EmacsFrame doesn't have a geometry but the shell does,
@@ -423,13 +434,13 @@ set_frame_size (ew)
     char shell_position [32];
 
     /* Take into account the size of the scrollbar */
-    frame->display.x->vertical_scroll_bar_extra
+    frame->output_data.x->vertical_scroll_bar_extra
       = (!FRAME_HAS_VERTICAL_SCROLL_BARS (frame)
 	 ? 0
 	 : FRAME_SCROLL_BAR_PIXEL_WIDTH (frame) > 0
 	 ? FRAME_SCROLL_BAR_PIXEL_WIDTH (frame)
 	 : (FRAME_SCROLL_BAR_COLS (frame)
-	    * FONT_WIDTH (frame->display.x->font)));
+	    * FONT_WIDTH (frame->output_data.x->font)));
 
     change_frame_size (frame, h, w, 1, 0);
     char_to_pixel_size (ew, w, h, &pixel_width, &pixel_height);
@@ -535,13 +546,13 @@ create_frame_gcs (ew)
 {
   struct frame *s = ew->emacs_frame.frame;
 
-  s->display.x->normal_gc
+  s->output_data.x->normal_gc
     = XCreateGC (XtDisplay (ew), RootWindowOfScreen (XtScreen (ew)),
 		 (unsigned long)0, (XGCValues *)0);
-  s->display.x->reverse_gc
+  s->output_data.x->reverse_gc
     = XCreateGC (XtDisplay (ew), RootWindowOfScreen (XtScreen (ew)),
 		 (unsigned long)0, (XGCValues *)0);
-  s->display.x->cursor_gc
+  s->output_data.x->cursor_gc
     = XCreateGC (XtDisplay (ew), RootWindowOfScreen (XtScreen (ew)),
 		 (unsigned long)0, (XGCValues *)0);
 }
@@ -593,7 +604,7 @@ setup_frame_gcs (ew)
   gc_values.graphics_exposures = False;
   gc_values.stipple = blank_stipple;
   gc_values.tile = blank_tile;
-  XChangeGC (XtDisplay (ew), s->display.x->normal_gc,
+  XChangeGC (XtDisplay (ew), s->output_data.x->normal_gc,
 	     (GCFont | GCForeground | GCBackground | GCGraphicsExposures
 	      | GCStipple | GCTile),
 	     &gc_values);
@@ -605,7 +616,7 @@ setup_frame_gcs (ew)
   gc_values.graphics_exposures = False;
   gc_values.stipple = blank_stipple;
   gc_values.tile = blank_tile;
-  XChangeGC (XtDisplay (ew), s->display.x->reverse_gc,
+  XChangeGC (XtDisplay (ew), s->output_data.x->reverse_gc,
 	     (GCFont | GCForeground | GCBackground | GCGraphicsExposures
 	      | GCStipple | GCTile),
 	     &gc_values);
@@ -620,7 +631,7 @@ setup_frame_gcs (ew)
     = XCreateBitmapFromData (XtDisplay (ew),
 			     RootWindowOfScreen (XtScreen (ew)),
 			     setup_frame_cursor_bits, 16, 16);
-  XChangeGC (XtDisplay (ew), s->display.x->cursor_gc,
+  XChangeGC (XtDisplay (ew), s->output_data.x->cursor_gc,
 	     (GCFont | GCForeground | GCBackground | GCGraphicsExposures
 	      | GCStipple | GCTile),
 	     &gc_values);
@@ -630,8 +641,8 @@ static void
 update_various_frame_slots (ew)
      EmacsFrame ew;
 {
-  struct x_display* x = ew->emacs_frame.frame->display.x;
-  x->pixel_height = ew->core.height;
+  struct x_output *x = ew->emacs_frame.frame->output_data.x;
+  x->pixel_height = ew->core.height + x->menubar_height;
   x->pixel_width = ew->core.width;
   x->internal_border_width = ew->emacs_frame.internal_border_width;
 
@@ -641,8 +652,8 @@ static void
 update_from_various_frame_slots (ew)
      EmacsFrame ew;
 {
-  struct x_display* x = ew->emacs_frame.frame->display.x;
-  ew->core.height = x->pixel_height;
+  struct x_output *x = ew->emacs_frame.frame->output_data.x;
+  ew->core.height = x->pixel_height - x->menubar_height;
   ew->core.width = x->pixel_width;
   ew->core.background_pixel = x->background_pixel;
   ew->emacs_frame.internal_border_width = x->internal_border_width;
@@ -702,7 +713,7 @@ EmacsFrameInitialize (request, new, dum1, dum2)
   }
 
 /* Update the font field in frame */
-  ew->emacs_frame.frame->display.x->font = ew->emacs_frame.font;
+  ew->emacs_frame.frame->output_data.x->font = ew->emacs_frame.font;
 #endif
 
   update_from_various_frame_slots (ew);
@@ -721,13 +732,8 @@ EmacsFrameRealize (widget, mask, attrs)
 {
   EmacsFrame ew = (EmacsFrame)widget;
 
-  attrs->event_mask = (KeyPressMask | ExposureMask | ButtonPressMask |
-		       ButtonReleaseMask | StructureNotifyMask |
-		       FocusChangeMask | PointerMotionHintMask |
-		       PointerMotionMask | LeaveWindowMask | EnterWindowMask |
-		       VisibilityChangeMask | PropertyChangeMask |
-		       StructureNotifyMask | SubstructureNotifyMask |
-		       SubstructureRedirectMask);
+  attrs->event_mask = (STANDARD_EVENT_SET | PropertyChangeMask
+		       | SubstructureNotifyMask | SubstructureRedirectMask);
   *mask |= CWEventMask;
   XtCreateWindow (widget, InputOutput, (Visual *)CopyFromParent, *mask,
 		  attrs);
@@ -744,8 +750,8 @@ EmacsFrameDestroy (widget)
   struct frame* s = ew->emacs_frame.frame;
 
   if (! s) abort ();
-  if (! s->display.x) abort ();
-  if (! s->display.x->normal_gc) abort ();
+  if (! s->output_data.x) abort ();
+  if (! s->output_data.x->normal_gc) abort ();
 
   /* this would be called from Fdelete_frame() but it needs to free some
      stuff after the widget has been finalized but before the widget has
@@ -754,9 +760,9 @@ EmacsFrameDestroy (widget)
 
   BLOCK_INPUT;
   /* need to be careful that the face-freeing code doesn't free these too */
-  XFreeGC (XtDisplay (widget), s->display.x->normal_gc);
-  XFreeGC (XtDisplay (widget), s->display.x->reverse_gc);
-  XFreeGC (XtDisplay (widget), s->display.x->cursor_gc);
+  XFreeGC (XtDisplay (widget), s->output_data.x->normal_gc);
+  XFreeGC (XtDisplay (widget), s->output_data.x->reverse_gc);
+  XFreeGC (XtDisplay (widget), s->output_data.x->cursor_gc);
   UNBLOCK_INPUT;
 }
 
@@ -770,7 +776,7 @@ EmacsFrameResize (widget)
   int rows;
 
   pixel_to_char_size (ew, ew->core.width, ew->core.height, &columns, &rows);
-  change_frame_size (f, rows, columns, 1, 0);
+  change_frame_size (f, rows, columns, 0, 1);
   update_wm_hints (ew); 
   update_various_frame_slots (ew);
 }
@@ -840,7 +846,7 @@ EmacsFrameSetValues (cur_widget, req_widget, new_widget, dum1, dum2)
      to wm_shell's iconic slot have no effect after it has been realized,
      and calling XIconifyWindow doesn't work either (even thought the window
      has been created.)  Perhaps there is some property we could smash
-     directly, but I'm sick of this for now.  Xt is a steaming pile of shit!
+     directly, but I'm sick of this for now.
    */
   if (cur->emacs_frame.iconic != new->emacs_frame.iconic)
     {
@@ -900,12 +906,12 @@ EmacsFrameSetCharSize (widget, columns, rows)
   if (columns < 3) columns = 3;  /* no way buddy */
 
   check_frame_size (f, &rows, &columns);
-  f->display.x->vertical_scroll_bar_extra
+  f->output_data.x->vertical_scroll_bar_extra
     = (!FRAME_HAS_VERTICAL_SCROLL_BARS (f)
        ? 0
        : FRAME_SCROLL_BAR_PIXEL_WIDTH (f) > 0
        ? FRAME_SCROLL_BAR_PIXEL_WIDTH (f)
-       : (FRAME_SCROLL_BAR_COLS (f) * FONT_WIDTH (f->display.x->font)));
+       : (FRAME_SCROLL_BAR_COLS (f) * FONT_WIDTH (f->output_data.x->font)));
 
   char_to_pixel_size (ew, columns, rows, &pixel_width, &pixel_height);
 
@@ -915,14 +921,14 @@ EmacsFrameSetCharSize (widget, columns, rows)
     {
       int hdelta = pixel_height - ew->core.height;
       int wdelta = pixel_width - ew->core.width;
-      int column_widget_height = f->display.x->column_widget->core.height;
-      int column_widget_width = f->display.x->column_widget->core.width;
-      int outer_widget_height = f->display.x->widget->core.height;
-      int outer_widget_width = f->display.x->widget->core.width;
-      int old_left = f->display.x->widget->core.x;
-      int old_top = f->display.x->widget->core.y;
+      int column_widget_height = f->output_data.x->column_widget->core.height;
+      int column_widget_width = f->output_data.x->column_widget->core.width;
+      int outer_widget_height = f->output_data.x->widget->core.height;
+      int outer_widget_width = f->output_data.x->widget->core.width;
+      int old_left = f->output_data.x->widget->core.x;
+      int old_top = f->output_data.x->widget->core.y;
 
-      lw_refigure_widget (f->display.x->column_widget, False);
+      lw_refigure_widget (f->output_data.x->column_widget, False);
       update_hints_inhibit = 1;
 
       ac = 0;
@@ -933,21 +939,23 @@ EmacsFrameSetCharSize (widget, columns, rows)
       ac = 0;
       XtSetArg (al[ac], XtNheight, column_widget_height + hdelta); ac++;
       XtSetArg (al[ac], XtNwidth, column_widget_width + wdelta); ac++;
-      XtSetValues (f->display.x->column_widget, al, ac);
+      XtSetValues (f->output_data.x->column_widget, al, ac);
 
       ac = 0;
       XtSetArg (al[ac], XtNheight, outer_widget_height + hdelta); ac++;
       XtSetArg (al[ac], XtNwidth, outer_widget_width + wdelta); ac++;
-      XtSetValues (f->display.x->widget, al, ac);
+      XtSetValues (f->output_data.x->widget, al, ac);
 
-      lw_refigure_widget (f->display.x->column_widget, True);
+      lw_refigure_widget (f->output_data.x->column_widget, True);
 
       update_hints_inhibit = 0;
       update_wm_hints (ew);
 
+      do_pending_window_change ();
+
       /* These seem to get clobbered.  I don't know why. - rms.  */
-      f->display.x->widget->core.x = old_left;
-      f->display.x->widget->core.y = old_top;
+      f->output_data.x->widget->core.x = old_left;
+      f->output_data.x->widget->core.y = old_top;
     }
 
   /* We've set {FRAME,PIXEL}_{WIDTH,HEIGHT} to the values we hope to
